<?php
/**
 * @copyright:	Wojtek Szałkiewicz
 * @license:	GPLv2 or later
 * 
 * This class is a base class for wordpress plugins.
 * It's a part of package in which you found it.
 * See readme.txt for more information.
 */

class EWPluginCore extends EWPlugable
{
	/**
	 * @var string  plugin name
	 */
	protected static $pluginName;

	/**
	 * @var string  plugin slug used in setting names etc.
	 */
	protected static $pluginSlug;

	/**
	 * @var string  plugin version
	 */
	protected static $version;

	/**
	 * @var string	plugin class name
	 */
	protected static $className = 'EasyWatermarkPlugin';

	/**
	 * @var string  main plugin file
	 */
	protected static $mainFile = 'index.php';

	/**
 	 * Initiates plugin by creating an object of inheriting class,
 	 * registers activation and uninstall hooks,
	 * checks version and executes upgrade function if needed.
	 *
	 * @return object
	 */
	public static function init(){
		global $ewClass;
		$ewClass = self::$className;
		// Register install and uninstall methods
		register_activation_hook(dirname(dirname(__FILE__)) . DIRECTORY_SEPARATOR . $ewClass::$mainFile, array($ewClass::$className, 'install'));
		register_uninstall_hook(dirname(dirname(__FILE__)) . DIRECTORY_SEPARATOR . $ewClass::$mainFile, array($ewClass::$className, 'uninstall'));

		$version = get_option($ewClass::$pluginSlug . '-version');
		if(!empty($version) & version_compare($version, $ewClass::$version, '<')){
			// Version from database is lower than current, upgrade...
			$ewClass::upgrade($version);
		}

		// Load plugin textdomain
		load_plugin_textdomain($ewClass::$pluginSlug, false, '/'.$ewClass::$pluginSlug.'/languages');

		// Create object of plugin class (inheritign this one)
		return new $ewClass();
	}

	/**
 	 * Returns plugin name
	 *
	 * @return string
	 */
	public static function getName(){
		global $ewClass;
		return $ewClass::$pluginName;
	}

	/**
 	 * Returns plugin slug
	 *
	 * @return string
	 */
	public static function getSlug(){
		global $ewClass;
		return $ewClass::$pluginSlug;
	}


	/**
 	 * Returns plugin version
	 *
	 * @return string
	 */
	public static function getVersion(){
		global $ewClass;
		return $ewClass::$version;
	}

	/**
 	 * Method run when activating plugin
	 *
	 * @return void
	 */
	public static function install(){}

	/**
 	 * Method run when removing plugin
	 *
	 * @return void
	 */
	public static function uninstall(){}

	/**
 	 * Method run when plugin version stored in WP options
	 * is lower than current version.
	 *
	 * @param  string  previously installed version
	 * @return void
	 */
	protected static function upgrade($version){}

	public function __call($name, $args){
		global $ewClass;
		if(method_exists($ewClass.'::'.$name)){
			return $ewClass::$name();
		}

		return false;
	}
}
